;(function(root, factory) {
  if (typeof define === 'function' && define.amd) {
    define(['jquery', 'packir'], factory);
  } else if (typeof define === 'function' && define.ajs) {
    define("jquery-packir", ['jquery', 'packir'], factory);
  } else if (typeof exports === 'object') {
    module.exports = factory(require('jquery'), require('packir'));
  } else {
    var _root = root;
    _root["jquery-packir"] = factory(root.jQuery, root.packir);
  }
}(this, function(_$, _packir) {
var require, define;
require=(function e(t,n,r){function s(o,u){if(!n[o]){if(!t[o]){var a=typeof require=="function"&&require;if(!u&&a)return a(o,!0);if(i)return i(o,!0);var f=new Error("Cannot find module '"+o+"'");throw f.code="MODULE_NOT_FOUND",f}var l=n[o]={exports:{}};t[o][0].call(l.exports,function(e){var n=t[o][1][e];return s(n?n:e)},l,l.exports,e,t,n,r)}return n[o].exports}var i=typeof require=="function"&&require;for(var o=0;o<r.length;o++)s(r[o]);return s})({1:[function(require,module,exports){
var inherits          = require('inherits-js'),
    packir            = _packir,
    BaseStrategy      = packir.BaseStrategy,
    FixedRowsStrategy = packir.FixedRowsStrategy,
    StackCompressor   = packir.StackCompressor,
    Rule              = packir.Rule,
    NonrecurringXRule = packir.NonrecurringXRule,
    NonrecurringYRule = packir.NonrecurringYRule,
    RecurringXRule    = packir.RecurringXRule,
    PreferredSizeRule = packir.PreferredSizeRule,
    Area              = packir.Area,
    _                 = require('./utils/utils');

var DATA_INSTANCE = "packir_instance";

/**
 * @class Packir
 * @constructor
 * @abstract
 *
 * @param {DOMElement} [$el]
 * @param {Object} [opts]
 */
function Packir($el, opts) {
    var options, strategy, compressor;

    this.$el = $el;
    this.options = options = _.extend({}, this.constructor.DEFAULTS, opts);

    this.state = {
        active: false
    };

    switch (options.strategy.type) {
        case "fixed-rows": {
            strategy = new FixedRowsStrategy(options.strategy.options);
            break;
        }

        default: {
            strategy = new BaseStrategy(options.strategy.options);
        }
    }

    switch (options.compressor) {
        default: {
            compressor = new StackCompressor();
        }
    }

    strategy.injectCompressor(compressor);

    _.forEach(options.rules, function(def) {
        var rule, options, r;

        if (_.isArray(def)) {
            rule = def[0];
            options = def[1];
        } else {
            rule = def;
        }

        switch (rule) {
            case "nonrecurring-y": {
                r = new NonrecurringYRule(options);
                break;
            }

            case "nonrecurring-x": {
                r = new NonrecurringXRule(options);
                break;
            }

            case "recurring-x": {
                r = new RecurringXRule(options);
                break;
            }

            case "preferred-size": {
                r = new PreferredSizeRule(options);
                break;
            }

            default: {
                if (rule instanceof Rule) {
                    return rule;
                }

                throw new Error('Unknown rule: "' + rule + '"');
            }
        }

        strategy.addRule(r);
    });

    this.strategy = strategy;

    this.hash = new packir.MurmurHash();
    this.store = new packir.LocalStore(options.store);

    this.blocks = $el.children().map(function(index) {
        var $clone;

        $clone = $(this).clone(true, true);

        return {
            width:  $clone.data(options.data.width)  || $clone.width(),
            height: $clone.data(options.data.height) || $clone.height(),
            weight: $clone.data(options.data.weight) || index,
            $el:    $clone
        };
    });

    this.children = $el.children().map(function(index) {
        return $(this).detach();
    });

    $el.data(DATA_INSTANCE, this);
}

Packir.prototype = {
    constructor: Packir,

    run: function(done) {
        var self = this,
            $el, area, options;

        options = this.options;

        $el = this.$el;

        area = new Area($el.width(), $el.height());

        if (area.width == 0) {
            throw new Error("Area width could not be non positive integer");
        }

        area.injectHash(this.hash);
        area.injectStore(this.store);
        area.injectStrategy(this.strategy);

        area.fill(this.blocks, options.options, function(err, grid) {
            var $grid;

            if (err) {
                done(err);
                return;
            }

            function X(v) {
                return v * grid.stepLeft;
            }

            function Y(v) {
                return v * grid.stepTop;
            }

            $grid = $("<div/>")
                .css({
                    width: X(grid.width) + 'px',
                    height: Y(grid.height) + 'px'
                })
                .addClass(options.className.grid);

            _.forEach(grid.rows, function(row, index, list) {
                var $row, lastRow, firstRow;

                firstRow = index == 0;
                lastRow = index == (list.length - 1);

                $row = $("<div/>")
                    .css({
                        width:  X(row.width) + 'px',
                        height: Y(row.height) + 'px',
                        top:    Y(row.position.top) + 'px',
                        left:   X(row.position.left) + 'px',
                        position: "absolute"
                    })
                    .addClass(options.className.row);

                if (firstRow) {
                    $row.addClass(options.className.rowTop);
                }

                if (lastRow) {
                    $row.addClass(options.className.rowBottom);
                }

                _.forEach(row.stacks, function(stack, index, list) {
                    var $stack, firstStack, lastStack;

                    firstStack = index == 0;
                    lastStack = index == (list.length - 1);

                    $stack = $("<div/>")
                        .css({
                            position: "absolute",
                            width:  X(stack.width) + 'px',
                            height: Y(stack.height) + 'px',
                            top:    Y(stack.position.top) + 'px',
                            left:   X(stack.position.left) + 'px'
                        })
                        .addClass(options.className.stack);

                    if (firstStack) {
                        $stack.addClass(options.className.stackLeft);
                    }

                    if (lastStack) {
                        $stack.addClass(options.className.stackRight);
                    }

                    _.forEach(stack.fields, function(field) {
                        var $field, mapping, $el, style;

                        mapping = self.blocks[field.id];
                        $el = mapping.$el;

                        $field = $("<div/>").addClass(options.className.field);
                        style = {
                            position: "absolute",
                            width:  X(field.width) + 'px',
                            height: Y(field.height) + 'px',
                            top:    Y(field.position.top) + 'px',
                            left:   X(field.position.left) + 'px',
                            overflow: 'hidden'
                        };

                        if (firstRow) {
                            $field.addClass(options.className.fieldTop);

                            if (options.className.elTop) {
                                $el.addClass(options.className.elTop);
                            }
                        }

                        if (lastRow) {
                            $field.addClass(options.className.fieldBottom);

                            if (options.className.elBottom) {
                                $el.addClass(options.className.elBottom);
                            }
                        }

                        if (firstStack) {
                            $field.addClass(options.className.fieldLeft);

                            if (options.className.elLeft) {
                                $el.addClass(options.className.elLeft);
                            }
                        }

                        if (lastStack) {
                            $field.addClass(options.className.fieldRight);

                            if (options.className.elRight) {
                                $el.addClass(options.className.elRight);
                            }
                        }

                        $field.css(style);

                        $field.append($el);
                        $stack.append($field);
                    });

                    $row.append($stack);
                });

                $grid.append($row);
            });

            self.reset();

            $el.append($grid);
            self.$grid = $grid;
            self.setState({
                active: true
            });

            done();
        });
    },

    reset: function() {
        if (this.state.active) {
            this.$grid.remove();
            this.setState({active: false});
        }
    },

    clear: function() {
        this.reset();

        _.forEach(this.children, function($el) {
            $el.appendTo(self.$el);
        });
    },

    setState: function(state) {
        _.extend(this.state, state);
    },

    method: function(name, options, cb) {
        switch (name) {
            case "reset": {
                this.reset();
                return cb();
            }

            default: {
                throw new TypeError("Unknown method '" + name + "'");
            }
        }
    }
};

Packir.DEFAULTS = {
    strategy: {
        type: "base",
        options: {}
    },
    compressor: "stack",
    rules: [
        "nonrecurring",
        "recurring"
    ],
    data: {
        width:  "width",
        height: "height",
        weight: "weight"
    },
    options: {
        margin: {
            x: 0,
            y: 0
        }
    },
    store: {
        ttl: 0
    },
    className: {
        grid:        "packir-grid",
        row:         "packir-row",
        rowTop:      "packir-row--top",
        rowBottom:   "packir-row--bottom",
        stack:       "packir-stack",
        stackLeft:   "packir-stack--left",
        stackRight:  "packir-stack--right",
        field:       "packir-field",
        fieldTop:    "packir-field--top",
        fieldRight:  "packir-field--right",
        fieldBottom: "packir-field--bottom",
        fieldLeft:   "packir-field--left",
        elTop:       null,
        elRight:     null,
        elBottom:    null,
        elLeft:      null
    }
};

Packir.find = function($el) {
    return $el.data(DATA_INSTANCE);
};

Packir.extend = function(proto, statics) {
    return inherits(this, proto, statics);
};

module.exports = Packir;
},{"./utils/utils":3,"inherits-js":4}],2:[function(require,module,exports){
var _ = require("./utils");

module.exports = function parallel(functions, done) {
    var wait, closed, complete;

    complete = 0;
    wait = functions.length;

    function reject(err) {
        if (!closed) {
            done(err);
            closed = true;
        }
    }

    function resolve() {
        if (!closed) {
            done();
            closed = true;
        }
    }

    _.forEach(functions, function(func) {
        if (closed) {
            return;
        }

        try {
            func.call(null, function(err) {
                if (err) {
                    reject(err);
                    return;
                }

                complete++;

                if (complete == wait) {
                    resolve();
                }
            })
        } catch (err) {
            reject(err);
            return;
        }
    });
};
},{"./utils":3}],3:[function(require,module,exports){
function forEachOwn(obj, iterator, context) {
    for (var x in obj) {
        if (obj.hasOwnProperty(x)) {
            iterator.call(context, obj[x], x, obj);
        }
    }
}

function forEachSimple(arr, iterator, context) {
	var len = arr.length;
	var x;

    if (exports.isFunction(arr.forEach)) {
        return arr.forEach(iterator, context);
    }

    for (x = 0; x < len; x++) {
        iterator.call(context, arr[x], x, arr);
    }
}

function reduce(obj, iterator, memo, context) {
    if (!obj) {
        return memo;
    }

    if (exports.isFunction(obj.reduce)) {
        return obj.reduce(iterator, memo, context);
    }

    exports.forEach(obj, function(value, index, list) {
        memo = iterator.call(context, memo, value, index, list);
    });

    return memo;
}

forEachSimple(["Object", "Array", "Function"], function(t) {
	exports["is" + t] = function(obj) {
		return Object.prototype.toString.call(obj) == "[object " + t + "]";	
	};
});

exports.noop = function(){};

exports.forEach = function(obj, iterator, ctx) {
	if (exports.isObject(obj)) {
		forEachOwn(obj, iterator, ctx);
	} else if (exports.isArray(obj)) {
		forEachSimple(obj, iterator, ctx);
	}

	return
};

exports.extend = function(dst) {
	if (!exports.isObject(dst)) {
		return dst;
	}

	forEachSimple(Array.prototype.slice.call(arguments, 1), function(source) {
        if (source) {
            forEachOwn(source, function(value, key) {
                obj[key] = value;
            });
        }
    });
};

exports.omit = function(obj, ignore) {
	var blacklist, b;

    if (exports.isFunction(ignore)) {
        blacklist = ignore;
    } else {
        if (exports.isArray(ignore)) {
            b = ignore;
        } else {
            b = Array.prototype.slice.call(arguments, 1);    
        }

        blacklist = function(value, key) {
            return b.indexOf(key) != -1;
        };
    }

    return reduce(obj, function(memo, value, key) {
        if (!blacklist(value, key)) {
            memo[key] = value;
        }

        return memo;
    }, {});
};
},{}],4:[function(require,module,exports){
var extend = require("./utils/extend");

module.exports = function(Parent, protoProps, staticProps) {
    var Child;

    protoProps  = protoProps  || {};
    staticProps = staticProps || {};

    if (protoProps.hasOwnProperty("constructor") && typeof protoProps.constructor === 'function') {
        Child = protoProps.constructor;
    } else {
        Child = function() {
            Parent.apply(this, arguments);
        };
    }

    // set the static props to the new Enum
    extend(Child, Parent, staticProps);

    // create prototype of Child, that created with Parent prototype
    //
    // __proto__  <----  __proto__
    //     ^                 ^
    //     |                 |
    //   Parent            Child
    //
    function Surrogate(){}
    Surrogate.prototype = Parent.prototype;
    Child.prototype = new Surrogate();

    // extend prototype
    extend(Child.prototype, protoProps);

    // set constructor directly
    // @see https://developer.mozilla.org/en-US/docs/ECMAScript_DontEnum_attribute#JScript_DontEnum_Bug
    Child.prototype.constructor = Child;


    return Child;
};
},{"./utils/extend":6}],5:[function(require,module,exports){
/**
 * Each iterator.
 *
 * @param {object}   obj
 * @param {function} func
 * @param {object}  [context]
 *
 * @returns {*}
 */
module.exports = function(obj, func, context) {
    var result;

    context || (context = null);

    for (var x in obj) {
        if (obj.hasOwnProperty(x)) {
            result = func.call(context, obj[x], x, obj);

            if (result !== undefined) {
                return result;
            }
        }
    }

    return result;
};
},{}],6:[function(require,module,exports){
var each = require("./each");

/**
 * Extends one object by multiple others.
 *
 * @param {object} to
 *
 * @returns {object}
 */
module.exports = function(to) {
    var from = Array.prototype.slice.call(arguments, 1);

    var func = function(value, prop) {
        to[prop] = value;
    };

    for (var x = 0; x < from.length; x++) {
        each(from[x], func);
    }

    return to;
};
},{"./each":5}],"jquery-packir":[function(require,module,exports){
var $        = _$,
    Packir   = require("./Packir"),
    _        = require("./utils/utils"),
    parallel = require("./utils/parallel"),
    _old;

_old = $.fn.packir;

$.fn.packir = function(method, options) {
    var done;

    if (_.isObject(method)) {
        options = method;
        method = null;
    }

    options = options || {};

    done = options.callback || _.noop;
    options = _.omit(options, "callback");

    parallel(
        this.map(function() {
            var $el = $(this),
                packir;

            if ( !(packir = Packir.find($el)) ) {
                if (method) {
                    throw new Error("Could not call method of non Packired element");
                }

                packir = new Packir($el, options);
            }

            if (method) {
                return function(next) {
                    packir.method(method, options, next);
                }
            }

            return function(next) {
                packir.run(next);
            }
        }),
        done
    );
};

$.fn.packir.noConflict = function() {
    $.fn.packir = _old;
};
},{"./Packir":1,"./utils/parallel":2,"./utils/utils":3}]},{},[]);

return require("jquery-packir");
}));