function forEachOwn(obj, iterator, context) {
    for (var x in obj) {
        if (obj.hasOwnProperty(x)) {
            iterator.call(context, obj[x], x, obj);
        }
    }
}

function forEachSimple(arr, iterator, context) {
	var len = arr.length;
	var x;

    if (exports.isFunction(arr.forEach)) {
        return arr.forEach(iterator, context);
    }

    for (x = 0; x < len; x++) {
        iterator.call(context, arr[x], x, arr);
    }
}

function reduce(obj, iterator, memo, context) {
    if (!obj) {
        return memo;
    }

    if (exports.isFunction(obj.reduce)) {
        return obj.reduce(iterator, memo, context);
    }

    exports.forEach(obj, function(value, index, list) {
        memo = iterator.call(context, memo, value, index, list);
    });

    return memo;
}

forEachSimple(["Object", "Array", "Function"], function(t) {
	exports["is" + t] = function(obj) {
		return Object.prototype.toString.call(obj) == "[object " + t + "]";	
	};
});

exports.noop = function(){};

exports.forEach = function(obj, iterator, ctx) {
	if (exports.isObject(obj)) {
		forEachOwn(obj, iterator, ctx);
	} else if (exports.isArray(obj)) {
		forEachSimple(obj, iterator, ctx);
	}

	return
};

exports.extend = function(dst) {
	if (!exports.isObject(dst)) {
		return dst;
	}

	forEachSimple(Array.prototype.slice.call(arguments, 1), function(source) {
        if (source) {
            forEachOwn(source, function(value, key) {
                obj[key] = value;
            });
        }
    });
};

exports.omit = function(obj, ignore) {
	var blacklist, b;

    if (exports.isFunction(ignore)) {
        blacklist = ignore;
    } else {
        if (exports.isArray(ignore)) {
            b = ignore;
        } else {
            b = Array.prototype.slice.call(arguments, 1);    
        }

        blacklist = function(value, key) {
            return b.indexOf(key) != -1;
        };
    }

    return reduce(obj, function(memo, value, key) {
        if (!blacklist(value, key)) {
            memo[key] = value;
        }

        return memo;
    }, {});
};